#!/usr/bin/env Rscript

###########################
#### Avis de copyright  ###
###########################

# Avis de copyright : le prsent code source a t dvelopp dans le cadre de l'enqute sur place et le rle de la recherche dans l'adaptation au changement climatique conduite par la Cour des comptes (2024). 
# Les lments de code source dvelopps par la Cour des comptes ont bnfici d'changes mthodologiques avec l'Agence nationale de la recherche pour un traitement harmonis de la casse (chanes de caractres) et une diversification des stratgies de recherche de mots cls.
# Ces lments sont proposs sous licence  GNU General Public License v3.0  (voir https://www.gnu.org/licenses/gpl-3.0.txt, ainsi que la copie intgrale et  l'identique de cette licence propose dans la prsente archive lectronique ouverte au public).

# Les donnes ouvertes de l'ANR employe comme cas d'application pour le prsent script sont proposes par l'Agence  cette adresse : https://www.data.gouv.fr/fr/organizations/agence-nationale-de-la-recherche/#/datasets

# Des exemples simples de mots cls drivs du thme du RPA 2024 de la Cour sont ici prsents pour fixer les ides. D'autres mots cls peuvent tre aisment ajouts par les utilisateurs. 

# Diffrentes sources peuvent tre mobilises pour enrichir les recherches smantiques. Voir par exemple : 
# - les mots cls tirs des glossaires des rapports du GIEC (voir fichier propos sur le site de la Cour rserv pour une utilisation non-commerciale)
# - l'tude Nalau et Verall (2021) "Mapping the evolution and current trends in climate change adaptation science", Climate Risk Management, https://doi.org/10.1016/j.crm.2021.100290


###########################
######## FUNCTIONS  #######
###########################

# harmonisation de la casse
harmonize <- function(char_vector){
  char_vector <- tolower(char_vector) # minuscules
  char_vector <- iconv(char_vector, from="UTF-8", to="ASCII//TRANSLIT//IGNORE") # sans accents
  char_vector <- gsub("[[:punct:]]", " ", char_vector) # sans ponctuation
  char_vector <- gsub("\\s+", " ", char_vector) # remove extra whitespaces
  char_vector <- trimws(char_vector) # remove leading and/or trailing whitespace
  return(char_vector)
}

# regexp pour rechercher toutes les variantes des mots tout en limitant le risque de faux positifs en imposant une contrainte sur la racine de ces mots
add_regexp_borders <- function(phrase_ou_mot){
  return(paste("\\b", phrase_ou_mot, "\\w*\\b", sep=""))
}

###########################
######## DATA #############
###########################

## Exemple d'application sur les projets financs par l'ANR (hors-PIA)
projets_ANR_DOS <- read.csv("OpenData/anr-dos-depuis-2010-projets-finances-20230313-projets.csv", sep = ";", encoding = "UTF-8")
colnames(projets_ANR_DOS)

## Hors-anne 2023 (incomplte  date)
projets_ANR_DOS <- projets_ANR_DOS[!(projets_ANR_DOS$AAP.Edition %in% 2023), ] 

## Selection des champs pour le filtrage 
focus_fields <- c("Projet.Acronyme",
                  "Projet.Titre.Francais",
                  "Projet.Titre.Anglais",
                  "Projet.Resume.Francais",
                  "Projet.Resume.Anglais")

## Harmonisation
projets_ANR_DOS[, focus_fields] <-  apply(projets_ANR_DOS[, focus_fields], 2, harmonize)

## Pooling des champs pour disposer d'un champ de filtrage unique
## Permet d'largir le champ de detection avec un risque de faux positifs rduit si les racines des mots sont bordes
## Le collapse = " ___ " (et non pas collapse = " " par ex.) permet de maintenir la possibilit d'une recherche par phrase stricte 
projets_ANR_DOS[, "champ_de_filtrage"] <- apply(projets_ANR_DOS[, focus_fields], 1, function(x) paste(x, collapse = " ___ "))

## Liste des lements recherchs avec diffrentes stratgies de recherche
## Ici application au thme du RPA 2024 de la Cour des comptes cibl sur l'adaptation au changement climatique
expressions <- list(

  ## Pour les recherche en franais, exemple de la phrase "adaptation au changement climatique" (et ses variantes)
  phrase_FR_exemple = c('\\w*adapt\\w*\\sau\\w*\\schang\\w*\\sclimat\\w*\\b'),
  racines_mots_FR_exemple = c("\\w*adapt\\w*\\sau\\w*\\b", add_regexp_borders(c("chang", "climat"))),
  caracteres_mots_FR_exemple = c("adapt\\w*\\sau", "chang", "climat"),
  
  ## La mme phrase traduite en anglais : "adaptation to climate change"
  phrase_EN_exemple = c('\\badapt\\w*\\sto\\sclimat\\w*\\schang\\w*\\b'), 
  racines_mots_EN_exemple = c("\\badapt\\w*\\sto\\b", add_regexp_borders(c("climat", "chang"))), # attention bordures spcifiques pour la premire partie
  caracteres_mots_EN_exemple = c("adapt\\w*\\sto", "climat", "chang") 
  
  )
  
## Verification d'unicit des noms des nouvelles colonnes
length(names(expressions)) = length(unique(names(expressions)))

## Ajout des nouvelles colonnes
projets_ANR_DOS[, names(expressions)] <- NA

## Recherche des mots cls et calcul des indicatrices associes
GT <- proc.time()
for (i in 1:length(names(expressions))){
  tt <- proc.time()
  projets_ANR_DOS[, names(expressions)[i]] <- ifelse(Reduce("&", lapply(expressions[[i]], function(x) grepl(x, projets_ANR_DOS$champ_de_filtrage))), 1, 0)
  print(proc.time() - tt)
}
print(proc.time() - GT)

## Ajout des colonnes de synthese
colnames(projets_ANR_DOS)

phrases_EN_FR <- colnames(projets_ANR_DOS)[grepl("phrase", colnames(projets_ANR_DOS))]
projets_ANR_DOS[, "phrases_EN_FR_total"] <- apply(projets_ANR_DOS[, phrases_EN_FR], 1, function(x) sum(x))
projets_ANR_DOS[, "phrases_EN_FR_binaire"] <- ifelse(projets_ANR_DOS[, "phrases_EN_FR_total"] >= 1, 1, 0)
print(sum(projets_ANR_DOS[, "phrases_EN_FR_binaire"]))

racines_mots_EN_FR <- colnames(projets_ANR_DOS)[grepl("racines_mots", colnames(projets_ANR_DOS))]
projets_ANR_DOS[, "racines_mots_EN_FR_total"] <- apply(projets_ANR_DOS[, racines_mots_EN_FR], 1, function(x) sum(x))
projets_ANR_DOS[, "racines_mots_EN_FR_binaire"] <- ifelse(projets_ANR_DOS[, "racines_mots_EN_FR_total"] >= 1, 1, 0)
print(sum(projets_ANR_DOS[, "racines_mots_EN_FR_binaire"]))

caracteres_mots_EN_FR <- colnames(projets_ANR_DOS)[grepl("caracteres_mots", colnames(projets_ANR_DOS))]
projets_ANR_DOS[, "caracteres_mots_EN_FR_total"] <- apply(projets_ANR_DOS[, caracteres_mots_EN_FR], 1, function(x) sum(x))
projets_ANR_DOS[, "caracteres_mots_EN_FR_binaire"] <- ifelse(projets_ANR_DOS[, "caracteres_mots_EN_FR_total"] >= 1, 1, 0)
print(sum(projets_ANR_DOS[, "caracteres_mots_EN_FR_binaire"]))